@@ -15,7 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 
 /* 
@@ -37,9 +38,9 @@
 #ifdef HAVE_BZIP2
 #include <bzlib.h>
 #endif /* HAVE_BZIP2 */
-#ifdef __riscos__
-# include <unixlib/local.h>
-#endif /* __riscos__ */
+#if defined(__riscos__) && defined(USE_ZLIBRISCOS)
+# include "zlib-riscos.h"
+#endif
 
 #define INCLUDED_BY_MAIN_MODULE 1
 #include "../g10/packet.h"
@@ -113,10 +114,6 @@ main( int argc, char **argv )
 {
   ARGPARSE_ARGS pargs;
 
-#ifdef __riscos__
-  /* set global RISC OS specific properties */
-  __riscosify_control = __RISCOSIFY_NO_PROCESS;
-#endif /* __riscos__ */
 #ifdef HAVE_DOSISH_SYSTEM
   setmode( fileno(stdin), O_BINARY );
   setmode( fileno(stdout), O_BINARY );
@@ -532,6 +529,12 @@ write_part ( const char *fname, FILE *fpin, unsigned long pktlen,
   unsigned char *p;
   const char *outname = create_filename (pkttype);
   
+#if defined(__riscos__) && defined(USE_ZLIBRISCOS)
+  static int initialized = 0;
+
+  if (!initialized)
+      initialized = riscos_load_module("ZLib", zlib_path, 1);
+#endif
   if (opt_no_split)
     fpout = stdout;
   else
